-- Simple Tranq Shot
-- 
-- This mod watches for the player to cast Tranquilizing Shot on
-- an enemy and sends a message to the desired channel based on if
-- the shot hits or misses the target.  This will catch any casts
-- of tranqilizing shot, regardless of how it's cast, but is
-- designed to remove the need for a macro to do the cast for you
-- preventing multiple spams of the broadcasted message in
-- situations where the shot is not actually cast.
--
-- All messages can be changed to the ones desired via console
-- commands, and the error message displayed on the ui when the
-- cast misses can be changed to any color.
--
-- /simpletranqshot
--

STranqVersion = 1.9
STranqTitle = "SimpleTranqShot "..STranqVersion;

STranq_Saved = { };

MissColor = { };
RaidMissColor = { };
STranq_ChanList = { };

STranq_Default = {
	["CastMsg"] = "-- casting tranq shot on %t --";
	["MissMsg"] = "== TRANQ SHOT MISSED %T ==";
	["MissErr"] = "TRANQUILIZING SHOT MISSED";
	["RaidErr"] = "%P MISSED TRANQUILIZING SHOT";
	["FailMsg"] = "== TRANQ SHOT FAILED ON %T ==";
	["FailErr"] = "TRANQUILIZING SHOT FAILED";
	["RaidFailErr"] = "%P'S TRANQUILIZING SHOT FAILED";
	["CastType"] = 1;
	["MissColor"] = { r = 1.0, g = 1.0, b = 0.0 };
	["RaidMissColor"] = { r = 1.0, g = 0.5, b = 0.0 };
};

STranq_Types = { "Yell", "Channel", "Raid", "Party", "Emote", "Say", "Whisper", "/rs" };

STranq_Mobs = {
	["Magmadar"] = {};
	["Death Talon Seether"] = {};
	["Chromaggus"] = {};
	["Flamegor"] = {};
	["Maws"] = {};
	["Princess Huhuran"] = {};
	["Vekniss Drone"] = {};
	["Vekniss Soldier"] = {};
	["Vekniss Drone"] = {};
	["Qiraji Slayer"] = {};
	["Roach"] = {};
	["Beetle"] = {};
	["Scorpion"] = {};
};

local STranq_string_Hunter = "Hunter";
local STranq_string_Tranq = "Tranquilizing Shot";
local STranq_string_SpellSelf = "You cast "..STranq_string_Tranq.." on (.+).";
local STranq_string_SpellOther = "(.+) casts "..STranq_string_Tranq.." on (.+)."
local STranq_string_SpellOtherMiss = "([^%s]+)'s "..STranq_string_Tranq.." missed (.+)";
local STranq_string_Frenzy = "Frenzy";
local STranq_string_FailSelf = "You fail to dispel (.+)'s "..STranq_string_Frenzy.."."
local STranq_string_FailOther = "(.+) fails to dispel (.+)'s "..STranq_string_Frenzy.."."
local STranq_string_GainFrenzy = "(.+) gains "..STranq_string_Frenzy.."."
local STranq_string_Removed = "(.+)'s "..STranq_string_Frenzy.." is removed."

local STranq_string_TestMob = "Test Mob";

STRANQ_BROADCAST_TEXT = "Broadcasted Messages";
STRANQ_ERROR_TEXT = "Error Messages";
STRANQ_CASTMSG_TEXT = "Successful Cast Message";
STRANQ_MISSMSG_TEXT = "Missed Cast Message";
STRANQ_MISSERR_TEXT = "Missed Cast Error";
STRANQ_RAIDERR_TEXT = "Raid Missed Cast Error";
STRANQ_FAILMSG_TEXT = "Failed Tranquilize Message";
STRANQ_FAILERR_TEXT = "Failed Tranquilize Error";
STRANQ_RAIDFAILERR_TEXT = "Raid Failed Tranquilize Error";
STRANQ_CASTTYPE_TEXT = "Cast Broadcast Type";
STRANQ_CASTCHAN_TEXT = "Channel";
STRANQ_CASTPLAY_TEXT = "Target Player";

STRANQ_NOCHANNEL_TEXT = "no channel";
STRANQ_NOPLAYER_TEXT = "no players";

STRANQ_MISSERRTEST_TEXT = "Test";
STRANQ_RAIDERRTEST_TEXT = "Test";
STRANQ_MISSCOLOR_TEXT = "Missed Cast Color";
STRANQ_RAIDCOLOR_TEXT = "Raid Missed Cast Color";
STRANQ_RAIDMISS_TEXT = "Show Raid Misses";
STRANQ_SOUNDS_TEXT = "Play Audio on Miss";
STRANQ_RS_TEXT = "Rebroadcast to /rs";

STRANQ_CASTMSG_TIP = "This message is broadcasted on a successful cast of Tranquilizing Shot.";
STRANQ_CASTMSGTEST_TIP = "Test successful cast message with selected settings.";
STRANQ_MISSMSG_TIP = "This message is broadcasted when Tranquilizing Shot misses.";
STRANQ_MISSMSGTEST_TIP = "Test missed cast message with selected settings.";
STRANQ_FAILMSG_TIP = "This message is broadcasted when Tranquilizing Shot fails to remove frenzy on the target it was cast on.";
STRANQ_FAILMSGTEST_TIP = "Test failed tranquilize message with selected settings.";

STRANQ_MISSERR_TIP = "This message is displayed when your Traquilizing Shot misses."
STRANQ_MISSERRTEST_TIP = "Test miss warning with selected settings.";
STRANQ_RAIDERR_TIP = "This message is displayed when someone else's Traquilizing Shot misses."
STRANQ_RAIDERRTEST_TIP = "Test raid miss warning with selected settings.";
STRANQ_FAILERR_TIP = "This message is displayed when your Traquilizing Shot fails."
STRANQ_FAILERRTEST_TIP = "Test failed tranquilize warning with selected settings.";
STRANQ_RAIDFAILERR_TIP = "This message is displayed when someone else's Traquilizing Shot fails."
STRANQ_RAIDFAILERRTEST_TIP = "Test raid failed tranquilize warning with selected settings.";

STRANQ_CASTTYPE_TIP = "Chat type to broadcast hit, miss, and fail messages to."
STRANQ_CASTCHAN_TIP = "Channel or player to broadcast to (if broadcast type requires it)."
STRANQ_RS_TIP = "Broadcast to CT_RaidAssist /rs (if available, otherwise /raid) as well as the selected broadcast type."

STRANQ_MISSCOLOR_TIP = "Change the color of the miss or fail warning."
STRANQ_RAIDCOLOR_TIP = "Change the color of the raid miss or fail warning."
STRANQ_RAIDMISS_TIP = "Display a warning when a near by raid member's Tranquilizing Shot fails or misses."
STRANQ_SOUND_TIP = "Play sound when warning on misses. Plays a sound for both raid misses and your own misses."

if ( GetLocale() == "frFR" ) then
	STranq_string_Hunter = "Chasseur";
	STranq_string_Tranq = "Tir tranquillisant"
	STranq_string_SpellSelf = "Vous lancez "..STranq_string_Tranq.." sur (.+).";
	STranq_string_SpellOther = "(.+) lance "..STranq_string_Tranq.." sur (.+).";
	STranq_string_SpellOtherMiss = "([^%s]+) voit son "..STranq_string_Tranq.." manquer (.+)";
	STranq_string_Frenzy = "fr\195\169n\195\169sie";
	STranq_string_FailSelf = "Vous n'avez pas russi  dissiper le "..STranq_string_Frenzy.." de (.+).";
	STranq_string_FailOther = "(.+) n'a pas russi  dissiper le "..STranq_string_Frenzy.." de (.+).";
	STranq_string_GainFrenzy = "(.+) gagne "..STranq_string_Frenzy..".";
	STranq_string_Removed = "(.+) n'est plus sous l'influence de "..STranq_string_Frenzy..".";
elseif ( GetLocale() == "deDE" ) then
	STranq_string_Hunter = "J\195\164ger";
	STranq_string_Tranq = "Einlullender Schuss";
	STranq_string_SpellSelf = "Ihr wirkt "..STranq_string_Tranq.." auf (.+).";
	STranq_string_SpellOther = "(.+) wirkt "..STranq_string_Tranq.." auf (.+).";
	STranq_string_SpellOtherMiss = STranq_string_Tranq.." von ([^%s]+) verfehlt (.+)";
	STranq_string_Frenzy = "Raserei";
	STranq_string_FailSelf = "(.+) kann dies nicht bannen: "..STranq_string_Frenzy..".";
	STranq_string_FailOther = "(.+) kann (.+)s "..STranq_string_Frenzy.." nicht bannen.";
	STranq_string_GainFrenzy = "(.+) bekommt "..STranq_string_Frenzy..".";
	STranq_string_Removed = "(.+)s "..STranq_string_Frenzy.." wurde entfernt.";
end

function STranq_OnLoad()
	this:RegisterEvent("PLAYER_ENTERING_WORLD")
end

--== Event Handler ==--
function STranq_OnEvent()
	if ( event == "CHAT_MSG_SPELL_SELF_DAMAGE" ) then
		if ( string.find( string.lower(arg1), string.lower(STranq_string_Tranq) ) ) then
			local text, miss;
			if ( string.find( arg1, STranq_string_SpellSelf ) ) then
				text = STranq_Name( CastMsg, UnitName("player"), UnitName("target") )
			else
				text = STranq_Name( MissMsg, UnitName("player"), UnitName("target") )
				miss = true;
				UIErrorsFrame:AddMessage( STranq_Name( MissErr, UnitName("player"), UnitName("target")), MissColor.r, MissColor.g, MissColor.b , 1.0, UIERRORS_HOLD_TIME)
				if ( not STranq_Saved["NoSound"] ) then
					PlaySoundFile("Sound\\Doodad\\BellTollHorde.wav");
				end
			end

			STranq_SendMessage( text );
		end
	elseif ( event == "CHAT_MSG_SPELL_PARTY_DAMAGE" or event == "CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE" ) then
		if ( GetNumRaidMembers() == 0 or STranq_Saved["HideRaidMisses"] ) then
			return; end

		if ( string.find( string.lower(arg1), string.lower(STranq_string_Tranq) ) ) then
			if ( not string.find( arg1, STranq_string_SpellOther ) ) then
				local _,_,name,mob = string.find( arg1, STranq_string_SpellOtherMiss );
				for i=1, GetNumRaidMembers(), 1 do
					if ( UnitName("raid"..i) == name ) then
						UIErrorsFrame:AddMessage( STranq_Name(RaidErr,name,mob), MissColor.r, MissColor.g, MissColor.b , 1.0, UIERRORS_HOLD_TIME)
						if ( not STranq_Saved["NoSound"] ) then
							PlaySoundFile("Sound\\Spells\\PVPFlagTaken.wav");
						end
						break;
					end
				end
			end
		end
	elseif ( event == "CHAT_MSG_SPELL_SELF_BUFF" ) then
		local _,_, mob = string.find( arg1, STranq_string_FailSelf );
		if ( mob ) then
			UIErrorsFrame:AddMessage( STranq_Name(FailErr,UnitName("player"),mob), MissColor.r, MissColor.g, MissColor.b , 1.0, UIERRORS_HOLD_TIME);
			if ( not STranq_Saved["NoSound"] ) then
				PlaySoundFile("Sound\\Doodad\\BellTollHorde.wav");
			end
			local text = STranq_Name( FailMsg, UnitName("player"), mob );
			STranq_SendMessage( text );
		end
	elseif ( event == "CHAT_MSG_SPELL_PARTY_BUFF" or event == "CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF" ) then
		if ( GetNumRaidMembers() == 0 or STranq_Saved["HideRaidMisses"] ) then
			return; end

		local _,_,name, mob = string.find( arg1, STranq_string_FailOther );
		if ( mob ) then
			for i=1, GetNumRaidMembers(), 1 do
				if ( UnitName("raid"..i) == name ) then
					UIErrorsFrame:AddMessage( arg1, MissColor.r, MissColor.g, MissColor.b , 1.0, UIERRORS_HOLD_TIME)
					if ( not STranq_Saved["NoSound"] ) then
						PlaySoundFile("Sound\\Spells\\PVPFlagTaken.wav");
					end
					break;
				end
			end
		end
	elseif ( event == "CHAT_MSG_SPELL_BREAK_AURA" ) then
		local _,_,mob = string.find( arg1, STranq_string_Removed );
		if ( mob ) then
			STranq_MobCheck( mob );
		end
	elseif ( event == "CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS" ) then
		local _,_,mob = string.find( arg1, STranq_string_GainFrenzy );
		if ( mob ) then
			STranq_MobCheck( mob );
		end	
	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		this:UnregisterEvent("PLAYER_ENTERING_WORLD")
		--if ( UnitClass("player") == STranq_string_Hunter ) then
			SlashCmdList["SIMPLETRANQ"] = STranq_Console;
			SLASH_SIMPLETRANQ1 = "/simpletranq";
			SLASH_SIMPLETRANQ2 = "/stranq";
			
			STranq_Load();
		--end
	end
end

function STranq_Print(text)
	if ( text ) then
		DEFAULT_CHAT_FRAME:AddMessage( "SimpleTranqShot |c99999999: "..text );
	end
end

function STranq_SendMessage(text)
	if ( STranq_Saved["RS"] or CastType == 8 ) then
		if ( CT_RA_Channel and CT_RA_Level > 0 ) then
			--if ( CastType == 8 and not ( STranq_Saved["RSMiss"] and miss ) ) then
				CT_RA_AddMessage("MS " .. text);
			--end
		elseif ( STranq_Saved["RS"] and not CastType == 8 ) then
			if ( CastType ~= 3 ) then
				SendChatMessage( text, "Raid" );
			end
		else
			--STranq_Print( "Cannot find CT_RaidAssist channel.  Yelling instead." );
			if ( CastType ~= 1 ) then
				SendChatMessage( text, "Yell" );
			end
		end
	end

	if ( CastType == 2 or CastType == 7 ) then
		local ChanIndex;
		if ( STranq_Saved["CastChan"] ) then
			if ( CastType == 7 ) then
				ChanIndex = STranq_Saved["CastChan"];
			else
				ChanIndex = GetChannelName(STranq_Saved["CastChan"]);
				if ( ChanIndex == 0 ) then
					ChanIndex = nil;
				end
			end
		end						
		if ( ChanIndex ) then
			SendChatMessage( text, STranq_Types[CastType], nil, ChanIndex );
		else
			if ( ChanIndex ) then
				STranq_Print( "Cast Message Broadcast Channel is not set.  Yelling instead.  Type /stranq and choose a channel." );
			else
				STranq_Print( "Cast Message Broadcast Channel |r"..STranq_Saved["CastChan"].."|c99999999 is not a currently joined channel.  Yelling instead." );
			end
			SendChatMessage( text, "Yell" );
		end
	elseif ( CastType < 8 ) then
		SendChatMessage( text, STranq_Types[CastType] );
	end
end

function STranq_Load()
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE");
	this:RegisterEvent("CHAT_MSG_SPELL_PARTY_DAMAGE");
	this:RegisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE");
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF");
	this:RegisterEvent("CHAT_MSG_SPELL_PARTY_BUFF");
	this:RegisterEvent("CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF"); -- "%1 fails to dispel %2's Frenzy."
	this:RegisterEvent("CHAT_MSG_SPELL_BREAK_AURA"); -- "%t Frenzy is removed."
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS"); -- "%1 gains Frenzy."
	
	tinsert(UISpecialFrames,"STranq_Options");
	
	STranq_Setup();
	DEFAULT_CHAT_FRAME:AddMessage( "SimpleTranqShot "..STranqVersion.." |c99999999: loaded :|r /stranq" );

	STranq_Options_CastType_DropDown.tooltip = STRANQ_CASTTYPE_TIP;
	STranq_Options_CastChan_DropDown.tooltip = STRANQ_CASTCHAN_TIP;
	STranq_Options_Color.tooltip = STRANQ_MISSCOLOR_TIP;
	STranq_Options_RaidColor.tooltip = STRANQ_RAIDCOLOR_TIP;
	STranq_Options_RaidMisses.tooltip = STRANQ_RAIDMISS_TIP;
	STranq_Options_Sounds.tooltip = STRANQ_SOUND_TIP;
	STranq_Options_RS.tooltip = STRANQ_RS_TIP;
end

--== Console command handler ==--
function STranq_Console(msg)
	local _,_,param = string.find(msg, "(%a+).+");
	if string.lower(msg) == "reset" then
		STranq_Options:ClearAllPoints();
		STranq_Options:SetPoint("CENTER",UIParent,"CENTER");
		STranq_Print( "Resetting to defaults" );
		STranq_Saved = { };
		STranq_Setup();
	else
		STranq_Options_Open();
	end
end	

--== Setup ==--
function STranq_Setup()
	if ( STranq_Saved["CastType"] ) then
		if ( not tonumber(STranq_Saved["CastType"]) ) then
			DEFAULT_CHAT_FRAME:AddMessage("upgrading variables"); -- upgrade from 1.4
			for i=1, 6 do
				if ( STranq_Types[i] == STranq_Saved["CastType"] ) then
					STranq_Saved["CastType"] = nil;
					STranq_Saved["CastType"] = i;
					break; 
				end
			end
		end
	end
	for k, v in STranq_Default do
		setglobal( k, (STranq_Saved[k] or STranq_Default[k]) )
	end
end

--== Known Valid Target Check ==--
function STranq_ValidTarget( msg )
	for k, v in STranq_Mobs do
		if ( string.find( msg, k ) and not STranq_Mobs[k].disabled ) then
			return true;
		end
	end
	if ( STranq_Saved["Mobs"] ) then
		for k, v in STranq_Saved["Mobs"] do
			if ( string.find( msg, k ) and not STranq_Saved["Mobs"].disabled ) then
				return true;
			end
		end
	end
end

--== Add to Valid Target list ==--
function STranq_MobCheck( mob )
	for k, v in STranq_Mobs do
		if ( mob == k ) then
			return;
		end
	end
	-- double check against already added list to prevent log spam
	if ( STranq_Saved["Mobs"] ) then
		for k, v in STranq_Saved["Mobs"] do
			if ( mob == k ) then
				return;
			end
		end
	end
	STranq_Print( "Found new mob that frenzies.  Adding "..mob.." to the list of known mobs.");
	if ( not STranq_Saved["Mobs"] ) then STranq_Saved["Mobs"] = {}; end
	STranq_Saved["Mobs"][mob] = {};
end

--== Name Sub ==--
function STranq_Name(text, name, mob)
	if ( mob ) then
		text = string.gsub( text, "%%t", mob );
		text = string.gsub( text, "%%T", string.upper(mob) );
	end
	if ( name ) then
		text = string.gsub( text, "%%p", name );
		text = string.gsub( text, "%%P", string.upper(name) );
	end
	return text;
end

--== Miss Color ==--
function STranq_MissColor()
		ColorPickerFrame.func = STranq_MissColorFunc;
		ColorPickerFrame.hasOpacity = nil;
		ColorPickerFrame.opacityFunc = nil;
		ColorPickerFrame.opacity = nil;
		ColorPickerFrame:SetColorRGB( STranq_Options.MissColor.r, STranq_Options.MissColor.g, STranq_Options.MissColor.b );
		ColorPickerFrame.previousValues = { r = STranq_Options.MissColor.r, g = STranq_Options.MissColor.g, b = STranq_Options.MissColor.b };
		ColorPickerFrame.cancelFunc = STranq_MissColorCancelFunc;
		ShowUIPanel(ColorPickerFrame);
end
function STranq_MissColorFunc()
	local newR, newG, newB = ColorPickerFrame:GetColorRGB();
	STranq_Options_ColorNormalTexture:SetVertexColor(newR, newG, newB);
	STranq_Options.MissColor = { r = newR, g = newG, b = newB };
end
function STranq_MissColorCancelFunc(previousValues)
	if ( previousValues.r ) then
		STranq_Options_ColorNormalTexture:SetVertexColor(previousValues.r, previousValues.g, previousValues.b);
		STranq_Options.MissColor = { r = previousValues.r, g = previousValues.g, b = previousValues.b };
	end
end

--== Raid Miss Color ==--
function STranq_RaidColor()
		ColorPickerFrame.func = STranq_RaidColorFunc;
		ColorPickerFrame.hasOpacity = nil;
		ColorPickerFrame.opacityFunc = nil;
		ColorPickerFrame.opacity = nil;
		ColorPickerFrame:SetColorRGB( STranq_Options.RaidMissColor.r, STranq_Options.RaidMissColor.g, STranq_Options.RaidMissColor.b );
		ColorPickerFrame.previousValues = { r = STranq_Options.RaidMissColor.r, g = STranq_Options.RaidMissColor.g, b = STranq_Options.RaidMissColor.b };
		ColorPickerFrame.cancelFunc = STranq_RaidColorCancelFunc;
		ShowUIPanel(ColorPickerFrame);
end
function STranq_RaidColorFunc()
	local newR, newG, newB = ColorPickerFrame:GetColorRGB();
	STranq_Options_RaidColorNormalTexture:SetVertexColor(newR, newG, newB);
	STranq_Options.RaidMissColor = { r = newR, g = newG, b = newB };
end
function STranq_RaidColorCancelFunc(previousValues)
	if ( previousValues.r ) then
		STranq_Options_RaidColorNormalTexture:SetVertexColor(previousValues.r, previousValues.g, previousValues.b);
		STranq_Options.RaidMissColor = { r = previousValues.r, g = previousValues.g, b = previousValues.b };
	end
end

--== Cast Msg Test ==--
function STranq_Options_Test_Msg(text)
	local savedid, savedchan, savedrs = STranq_Saved["CastType"], STranq_Saved["CastChan"], STranq_Saved["RS"];

	local id = UIDropDownMenu_GetSelectedID(STranq_Options_CastType_DropDown);
	if ( id == 2 or id == 7 ) then
		local newChan = STranq_Options_CastChan_DropDownText:GetText();
		if ( newChan and not ( newChan == STRANQ_NOCHANNEL_TEXT or newChan == STRANQ_NOPLAYER_TEXT ) ) then
			STranq_Saved["CastChan"] = STranq_Options_CastChan_DropDownText:GetText();
			CastType = id;
		else
			message( "\n\nNo valid chat channel or player found." );
			return;
		end
	else
		STranq_Saved["CastChan"] = nil;
		CastType = id;
	end
	STranq_Saved["RS"] = STranq_Options_RS:GetChecked();
	
	STranq_SendMessage( STranq_Name( text, UnitName("player"), STranq_string_TestMob) );
	
	CastType = savedid;
	STranq_Saved["CastChan"] = savedchan;
	STranq_Saved["RS"] = savedrs;
end

--== Self Error Test ==--
function STranq_Options_Test_SelfErr(text)
	UIErrorsFrame:AddMessage( STranq_Name( text, UnitName("player"), STranq_string_TestMob), STranq_Options.MissColor.r, STranq_Options.MissColor.g, STranq_Options.MissColor.b , 1.0, UIERRORS_HOLD_TIME);
	if ( STranq_Options_Sounds:GetChecked() ) then
		PlaySoundFile("Sound\\Doodad\\BellTollHorde.wav");
	end
end

--== Raid Error Test ==--
function STranq_Options_Test_RaidErr(text)
	UIErrorsFrame:AddMessage( STranq_Name( text, UnitName("player"), STranq_string_TestMob), STranq_Options.RaidMissColor.r, STranq_Options.RaidMissColor.g, STranq_Options.RaidMissColor.b , 1.0, UIERRORS_HOLD_TIME);
	if ( STranq_Options_Sounds:GetChecked() ) then
		PlaySoundFile("Sound\\Spells\\PVPFlagTaken.wav");
	end
end

--== Cast Types Dropdown ==--
function STranq_Options_CastType_DropDown_OnLoad()
	UIDropDownMenu_Initialize(this, STranq_Options_CastType_DropDown_Initialize);
	UIDropDownMenu_SetWidth(155);
	UIDropDownMenu_SetSelectedID(STranq_Options_CastType_DropDown, 1);
end
function STranq_Options_CastType_DropDown_Initialize()
	for k, v in STranq_Types do
		if ( k == 8 and not IsAddOnLoaded("CT_RaidAssist") ) then
			break;
		end
		local info = {};
		info.text = STranq_Types[k];
		info.func = STranq_Options_CastType_DropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end
function STranq_Options_CastType_DropDown_OnClick()
	UIDropDownMenu_SetSelectedID(STranq_Options_CastType_DropDown, this:GetID());
	if ( this:GetID() == 2 or this:GetID() == 7 ) then
		UIDropDownMenu_Initialize(STranq_Options_CastChan_DropDown, STranq_Options_CastChan_DropDown_Initialize);
		if ( STranq_Saved["CastChan"] and GetChannelName(STranq_Saved["CastChan"]) > 0 ) then
			UIDropDownMenu_SetSelectedName(STranq_Options_CastChan_DropDown, STranq_Saved["CastChan"]);
		else
			UIDropDownMenu_SetSelectedID(STranq_Options_CastChan_DropDown, 1);
			if ( getn(STranq_ChanList) > 0 ) then
				STranq_Options_CastChan_DropDownText:SetTextColor(1, 1, 1);
			else
				if ( this:GetID() == 2 ) then
					STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOCHANNEL_TEXT);
				else
					STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOPLAYER_TEXT);
				end
				STranq_Options_CastChan_DropDownText:SetTextColor(0.3, 0.3, 0.3);
			end
		end
		STranq_Options_CastChan_Text:SetTextColor(1, 1, 1);
		STranq_Options_CastChan_DropDownButton:Enable();
	else
		STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOCHANNEL_TEXT);
		STranq_Options_CastChan_Text:SetText(STRANQ_CASTCHAN_TEXT);
		STranq_Options_CastChan_Text:SetTextColor(0.3, 0.3, 0.3);
		STranq_Options_CastChan_DropDownButton:Disable();
		STranq_Options_CastChan_DropDownText:SetTextColor(0.3, 0.3, 0.3);
	end
	if ( this:GetID() == 8 or not IsAddOnLoaded("CT_RaidAssist") ) then
		STranq_Options_RS:Disable()
		STranq_Options_RS_Text:SetTextColor(0.3, 0.3, 0.3);
	else
		STranq_Options_RS:Enable()
		STranq_Options_RS_Text:SetTextColor(1, 1, 1);
	end
end

--== Cast Channel Drowndown ==--
function STranq_Options_CastChan_DropDown_OnLoad()
	UIDropDownMenu_Initialize(this, STranq_Options_CastChan_DropDown_Initialize);
	UIDropDownMenu_SetWidth(155);
	if ( STranq_Saved["CastChan"] and GetChannelName(STranq_Saved["CastChan"]) ) then
		UIDropDownMenu_SetSelectedID(STranq_Options_CastChan_DropDown, GetChannelName(STranq_Saved["CastChan"]));
	else
		UIDropDownMenu_SetSelectedID(STranq_Options_CastChan_DropDown, 1);
	end
end
function STranq_Options_CastChan_DropDown_Initialize()
	 STranq_ChannelCheck();
	
	if ( getn(STranq_ChanList) > 0 ) then
		for k, v in STranq_ChanList do
			local info = {};
			info.text = STranq_ChanList[k];
			info.func = STranq_Options_CastChan_DropDown_OnClick;
			UIDropDownMenu_AddButton(info);
		end
	else
		local id = UIDropDownMenu_GetSelectedID(STranq_Options_CastType_DropDown);
		if ( id == 2 ) then
			STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOCHANNEL_TEXT);
		else
			STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOPLAYER_TEXT);
		end
		STranq_Options_CastChan_DropDownText:SetTextColor(0.3, 0.3, 0.3);
	end
end
function STranq_ChannelCheck()
	STranq_ChanList = { };
	if ( UIDropDownMenu_GetSelectedID(STranq_Options_CastType_DropDown) == 2 ) then
		local channelList = {GetChannelList()};
		local serverChanList = {EnumerateServerChannels()};	
		if ( channelList and serverChanList ) then
			for k, v in channelList, 2 do
				local cullMe;
				local chan = channelList[k+1];
				for j, w in serverChanList do
					if ( chan == serverChanList[j] ) then
						cullMe = true;
					end
				end
				if ( CT_RA_Channel and chan == CT_RA_Channel ) then
					cullMe = true;
				end
				if ( DamageMeters_syncChannel and chan == DamageMeters_syncChannel ) then
					cullMe = true;
				end
				if ( Sky ) then
					if ( SkyChannelManager.isSkyChannel( chan ) ) then
						cullMe = true;
					end
				end
				if ( not cullMe ) then
					tinsert(STranq_ChanList, chan);
				end
				k = k+1;
			end
		end
	else
		for i=1, GetNumRaidMembers() do
			if ( UnitClass("raid"..i ) == STranq_string_Hunter and not UnitIsUnit("raid"..i, "player") ) then
				tinsert(STranq_ChanList, UnitName("raid"..i));
			end
		end
	end
end
function STranq_Options_CastChan_DropDown_OnClick()
	STranq_Options_CastChan_DropDownText:SetTextColor(1, 1, 1);
	UIDropDownMenu_SetSelectedID(STranq_Options_CastChan_DropDown, this:GetID());
end

--== Options Menu Open ==--
function STranq_Options_Open()
	STranq_Options_CastMsg:SetText(CastMsg);
	STranq_Options_MissMsg:SetText(MissMsg);
	STranq_Options_MissErr:SetText(MissErr);
	STranq_Options_RaidErr:SetText(RaidErr);
	STranq_Options_FailMsg:SetText(FailMsg);
	STranq_Options_FailErr:SetText(FailErr);
	STranq_Options_RaidFailErr:SetText(RaidFailErr);
	STranq_Options_ColorNormalTexture:SetVertexColor(MissColor.r, MissColor.g, MissColor.b);
	STranq_Options.MissColor = MissColor;
	STranq_Options_RaidColorNormalTexture:SetVertexColor(RaidMissColor.r, RaidMissColor.g, RaidMissColor.b);
	STranq_Options.RaidMissColor = RaidMissColor;
	STranq_Options_RaidMisses:SetChecked( not STranq_Saved["RaidMisses"] );
	STranq_Options_Sounds:SetChecked( not STranq_Saved["NoSound"] );
	STranq_Options_RS:SetChecked( STranq_Saved["RS"] );
	UIDropDownMenu_SetSelectedID(STranq_Options_CastType_DropDown, CastType);
	STranq_Options_CastType_DropDownText:SetText(STranq_Types[CastType]);
	if ( CastType == 2 or CastType == 7 ) then
		UIDropDownMenu_Initialize(STranq_Options_CastChan_DropDown, STranq_Options_CastChan_DropDown_Initialize);
		if ( STranq_Saved["CastChan"] and GetChannelName(STranq_Saved["CastChan"]) > 0 ) then
			UIDropDownMenu_SetSelectedName(STranq_Options_CastChan_DropDown, STranq_Saved["CastChan"]);
		else
			UIDropDownMenu_SetSelectedID(STranq_Options_CastChan_DropDown, 1);
		end
		STranq_Options_CastChan_Text:SetTextColor(1, 1, 1);
		STranq_Options_CastChan_DropDownButton:Enable();
		if ( getn(STranq_ChanList) > 0 ) then
			STranq_Options_CastChan_DropDownText:SetTextColor(1, 1, 1);
		else
			if ( CastType == 2 ) then
				STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOCHANNEL_TEXT);
			else
				STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOPLAYER_TEXT);
			end
			STranq_Options_CastChan_DropDownText:SetTextColor(0.3, 0.3, 0.3);
		end
	else
		UIDropDownMenu_SetSelectedID(STranq_Options_CastChan_DropDown, 1);
		STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOCHANNEL_TEXT);
		STranq_Options_CastChan_Text:SetTextColor(0.3, 0.3, 0.3);
		STranq_Options_CastChan_DropDownButton:Disable();
		STranq_Options_CastChan_DropDownText:SetTextColor(0.3, 0.3, 0.3);
	end
	if ( CastType == 8 or not IsAddOnLoaded("CT_RaidAssist") ) then
		STranq_Options_RS:Disable()
		STranq_Options_RS_Text:SetTextColor(0.3, 0.3, 0.3);
	else
		STranq_Options_RS:Enable()
		STranq_Options_RS_Text:SetTextColor(1, 1, 1);
	end
	ShowUIPanel(STranq_Options);
end

--== Set Defaults ==--
function STranq_Options_Defaults()
	STranq_Options_CastMsg:SetText(STranq_Default["CastMsg"]);
	STranq_Options_MissMsg:SetText(STranq_Default["MissMsg"]);
	STranq_Options_MissErr:SetText(STranq_Default["MissErr"]);
	STranq_Options_RaidErr:SetText(STranq_Default["RaidErr"]);
	STranq_Options_RaidErr:SetText(STranq_Default["RaidErr"]);
	STranq_Options_FailMsg:SetText(STranq_Default["FailMsg"]);
	STranq_Options_FailErr:SetText(STranq_Default["FailErr"]);
	STranq_Options_RaidFailErr:SetText(STranq_Default["RaidFailErr"]);
	STranq_Options_ColorNormalTexture:SetVertexColor(STranq_Default["MissColor"].r, STranq_Default["MissColor"].g, STranq_Default["MissColor"].b);
	STranq_Options.MissColor = STranq_Default["MissColor"];
	STranq_Options_RaidColorNormalTexture:SetVertexColor(STranq_Default["RaidMissColor"].r, STranq_Default["RaidMissColor"].g, STranq_Default["RaidMissColor"].b);
	STranq_Options.RaidMissColor = STranq_Default["RaidMissColor"];
	STranq_Options_RaidMisses:SetChecked( 1 );
	STranq_Options_Sounds:SetChecked( 1 );
	STranq_Options_RS:SetChecked( nil );
	UIDropDownMenu_SetSelectedID(STranq_Options_CastType_DropDown, 1);
	UIDropDownMenu_SetSelectedID(STranq_Options_CastChan_DropDown, 1);
	STranq_Options_CastType_DropDownText:SetText("Yell");
	STranq_Options_CastChan_DropDownText:SetText(STRANQ_NOCHANNEL_TEXT);
	STranq_Options_CastChan_Text:SetTextColor(0.3, 0.3, 0.3);
	STranq_Options_CastChan_DropDownButton:Disable();
	STranq_Options_CastChan_DropDownText:SetTextColor(0.3, 0.3, 0.3);
end

--== Save Options ==--
function STranq_Options_Save()
	local id = UIDropDownMenu_GetSelectedID(STranq_Options_CastType_DropDown);
	if ( id == 2 or id == 7 ) then
		--STranq_Saved["CastChan"] = UIDropDownMenu_GetSelectedName(STranq_Options_CastChan_DropDown);
		local newChan = STranq_Options_CastChan_DropDownText:GetText();
		if ( newChan and not ( newChan == STRANQ_NOCHANNEL_TEXT or newChan == STRANQ_NOPLAYER_TEXT ) ) then
			STranq_Saved["CastChan"] = STranq_Options_CastChan_DropDownText:GetText();
			STranq_Saved["CastType"] = id;
		else
			message( "\n\nNo valid chat channel or player found." );
			return;
		end
	else
		STranq_Saved["CastChan"] = nil;
		STranq_Saved["CastType"] = id;
	end
	STranq_Saved["CastMsg"] = STranq_Options_CastMsg:GetText();
	STranq_Saved["MissMsg"] = STranq_Options_MissMsg:GetText();
	STranq_Saved["MissErr"] = STranq_Options_MissErr:GetText();
	STranq_Saved["RaidErr"] = STranq_Options_RaidErr:GetText();
	STranq_Saved["FailMsg"] = STranq_Options_FailMsg:GetText();
	STranq_Saved["FailErr"] = STranq_Options_FailErr:GetText();
	STranq_Saved["RaidFailErr"] = STranq_Options_RaidFailErr:GetText();
	STranq_Saved["RaidMisses"] = not STranq_Options_RaidMisses:GetChecked();
	STranq_Saved["NoSound"] = not STranq_Options_Sounds:GetChecked();
	STranq_Saved["RS"] = STranq_Options_RS:GetChecked();
	STranq_Saved["MissColor"] = {};
	STranq_Saved["MissColor"].r = STranq_Options.MissColor.r;
	STranq_Saved["MissColor"].g = STranq_Options.MissColor.g;
	STranq_Saved["MissColor"].b = STranq_Options.MissColor.b;
	STranq_Saved["RaidMissColor"] = {};
	STranq_Saved["RaidMissColor"].r = STranq_Options.RaidMissColor.r;
	STranq_Saved["RaidMissColor"].g = STranq_Options.RaidMissColor.g;
	STranq_Saved["RaidMissColor"].b = STranq_Options.RaidMissColor.b;
	STranq_Setup();
	STranq_Options_Close()
end

function STranq_Options_Close()
	HideUIPanel(STranq_Options);
end